@@ -3,7 +3,7 @@ use strict;
 use warnings;
 package Capture::Tiny;
 # ABSTRACT: Capture STDOUT and STDERR from Perl, XS or external programs
-our $VERSION = '0.16'; # VERSION
+our $VERSION = '0.18'; # VERSION
 use Carp ();
 use Exporter ();
 use IO::Handle ();
@@ -162,19 +162,22 @@ sub _unproxy {
 }
 
 sub _copy_std {
-  my %handles = map { $_, IO::Handle->new } qw/stdin stdout stderr/;
-  # _debug( "# copying std handles ...\n" );
-  _open $handles{stdin},   "<&STDIN";
-  _open $handles{stdout},  ">&STDOUT";
-  _open $handles{stderr},  ">&STDERR";
+  my %handles;
+  for my $h ( qw/stdout stderr stdin/ ) {
+    next if $h eq 'stdin' && ! $IS_WIN32; # WIN32 hangs on tee without STDIN copied
+    my $redir = $h eq 'stdin' ? "<&" : ">&";
+    _open $handles{$h} = IO::Handle->new(), $redir . uc($h); # ">&STDOUT" or "<&STDIN"
+  }
   return \%handles;
 }
 
+# In some cases we open all (prior to forking) and in others we only open
+# the output handles (setting up redirection)
 sub _open_std {
   my ($handles) = @_;
-  _open \*STDIN, "<&" . fileno $handles->{stdin};
-  _open \*STDOUT, ">&" . fileno $handles->{stdout};
-  _open \*STDERR, ">&" . fileno $handles->{stderr};
+  _open \*STDIN, "<&" . fileno $handles->{stdin} if defined $handles->{stdin};
+  _open \*STDOUT, ">&" . fileno $handles->{stdout} if defined $handles->{stdout};
+  _open \*STDERR, ">&" . fileno $handles->{stderr} if defined $handles->{stderr};
 }
 
 #--------------------------------------------------------------------------#
@@ -182,13 +185,11 @@ sub _open_std {
 #--------------------------------------------------------------------------#
 
 sub _start_tee {
-  my ($which, $stash) = @_;
+  my ($which, $stash) = @_; # $which is "stdout" or "stderr"
   # setup pipes
   $stash->{$_}{$which} = IO::Handle->new for qw/tee reader/;
   pipe $stash->{reader}{$which}, $stash->{tee}{$which};
-  # _debug( "# pipe for $which\: " .  _name($stash->{tee}{$which}) . " "
-  #  . fileno( $stash->{tee}{$which} ) . " => " . _name($stash->{reader}{$which})
-  #  . " " . fileno( $stash->{reader}{$which}) . "\n" );
+  # _debug( "# pipe for $which\: " .  _name($stash->{tee}{$which}) . " " . fileno( $stash->{tee}{$which} ) . " => " . _name($stash->{reader}{$which}) . " " . fileno( $stash->{reader}{$which}) . "\n" );
   select((select($stash->{tee}{$which}), $|=1)[0]); # autoflush
   # setup desired redirection for parent and child
   $stash->{new}{$which} = $stash->{tee}{$which};
@@ -206,12 +207,8 @@ sub _start_tee {
     # _debug( "# Win32API::File loaded\n") unless $@;
     my $os_fhandle = GetOsFHandle( $stash->{tee}{$which} );
     # _debug( "# Couldn't get OS handle: " . fileLastError() . "\n") if ! defined $os_fhandle || $os_fhandle == INVALID_HANDLE_VALUE();
-    if ( SetHandleInformation( $os_fhandle, HANDLE_FLAG_INHERIT(), 0) ) {
-      # _debug( "# set no-inherit flag on $which tee\n" );
-    }
-    else {
-      # _debug( "# can't disable tee handle flag inherit: " . fileLastError() . "\n");
-    }
+    my $result = SetHandleInformation( $os_fhandle, HANDLE_FLAG_INHERIT(), 0);
+    # _debug( $result ? "# set no-inherit flag on $which tee\n" : ("# can't disable tee handle flag inherit: " . fileLastError() . "\n"));
     _open_std( $stash->{child}{$which} );
     $stash->{pid}{$which} = system(1, @cmd, $stash->{flag_files}{$which});
     # not restoring std here as it all gets redirected again shortly anyway
@@ -222,7 +219,7 @@ sub _start_tee {
 }
 
 sub _fork_exec {
-  my ($which, $stash) = @_;
+  my ($which, $stash) = @_; # $which is "stdout" or "stderr"
   my $pid = fork;
   if ( not defined $pid ) {
     Carp::confess "Couldn't fork(): $!";
@@ -316,6 +313,7 @@ sub _capture_tee {
     if tied(*STDERR) && (reftype tied *STDERR eq 'GLOB');
   # _debug( "# tied object corrected layers for $_\: @{$layers{$_}}\n" ) for qw/stdin stdout stderr/;
   # bypass scalar filehandles and tied handles
+  # localize scalar STDIN to get a proxy to pick up FD0, then restore later to CT_ORIG_STDIN
   my %localize;
   $localize{stdin}++,  local(*STDIN)
     if grep { $_ eq 'scalar' } @{$layers{stdin}};
@@ -323,6 +321,8 @@ sub _capture_tee {
     if $do_stdout && grep { $_ eq 'scalar' } @{$layers{stdout}};
   $localize{stderr}++, local(*STDERR)
     if ($do_stderr || $do_merge) && grep { $_ eq 'scalar' } @{$layers{stderr}};
+  $localize{stdin}++, local(*STDIN), _open( \*STDIN, "<&=0")
+    if tied *STDIN && $] >= 5.008;
   $localize{stdout}++, local(*STDOUT), _open( \*STDOUT, ">&=1")
     if $do_stdout && tied *STDOUT && $] >= 5.008;
   $localize{stderr}++, local(*STDERR), _open( \*STDERR, ">&=2")
@@ -332,7 +332,6 @@ sub _capture_tee {
   my %proxy_std = _proxy_std();
   # _debug( "# proxy std: @{ [%proxy_std] }\n" );
   # update layers after any proxying
-  $layers{stdin}  = [PerlIO::get_layers(\*STDIN)]  if $proxy_std{stdin};
   $layers{stdout} = [PerlIO::get_layers(\*STDOUT)] if $proxy_std{stdout};
   $layers{stderr} = [PerlIO::get_layers(\*STDERR)] if $proxy_std{stderr};
   # _debug( "# post-proxy layers for $_\: @{$layers{$_}}\n" ) for qw/stdin stdout stderr/;
@@ -372,21 +371,25 @@ sub _capture_tee {
   _unproxy( %proxy_std );
   # _debug( "# killing tee subprocesses ...\n" ) if $do_tee;
   _kill_tees( $stash ) if $do_tee;
-  # return captured output
+  # return captured output, but shortcut in void context
+  # unless we have to echo output to tied/scalar handles;
   my %got;
-  for ( keys %do ) {
-    _relayer($stash->{capture}{$_}, $layers{$_});
-    $got{$_} = _slurp($_, $stash);
-    # _debug("# slurped " . length($got{$_}) . " bytes from $_\n");
+  if ( defined wantarray or ($do_tee && keys %localize) ) {
+    for ( keys %do ) {
+      _relayer($stash->{capture}{$_}, $layers{$_});
+      $got{$_} = _slurp($_, $stash);
+      # _debug("# slurped " . length($got{$_}) . " bytes from $_\n");
+    }
+    print CT_ORIG_STDOUT $got{stdout}
+      if $do_stdout && $do_tee && $localize{stdout};
+    print CT_ORIG_STDERR $got{stderr}
+      if $do_stderr && $do_tee && $localize{stderr};
   }
-  print CT_ORIG_STDOUT $got{stdout}
-    if $do_stdout && $do_tee && $localize{stdout};
-  print CT_ORIG_STDERR $got{stderr}
-    if $do_stderr && $do_tee && $localize{stderr};
   $? = $exit_code;
   $@ = $inner_error if $inner_error;
   die $outer_error if $outer_error;
   # _debug( "# ending _capture_tee with (@_)...\n" );
+  return unless defined wantarray;
   my @return;
   push @return, $got{stdout} if $do_stdout;
   push @return, $got{stderr} if $do_stderr;
@@ -406,7 +409,7 @@ Capture::Tiny - Capture STDOUT and STDERR from Perl, XS or external programs
 
 =head1 VERSION
 
-version 0.16
+version 0.18
 
 =head1 SYNOPSIS
 
@@ -433,7 +436,7 @@ version 0.16
 Capture::Tiny provides a simple, portable way to capture almost anything sent
 to STDOUT or STDERR, regardless of whether it comes from Perl, from XS code or
 from an external program.  Optionally, output can be teed so that it is
-captured while being passed through to the original handles.  Yes, it even
+captured while being passed through to the original filehandles.  Yes, it even
 works on Windows (usually).  Stop guessing which of a dozen capturing modules
 to use in any particular situation and just use this one.
 
@@ -449,8 +452,8 @@ The following functions are available.  None are exported by default.
 The C<<< capture >>> function takes a code reference and returns what is sent to
 STDOUT and STDERR as well as any return values from the code reference.  In
 scalar context, it returns only STDOUT.  If no output was received for a
-handle, it returns an empty string for that handle.  Regardless of calling
-context, all output is captured -- nothing is passed to the existing handles.
+filehandle, it returns an empty string for that filehandle.  Regardless of calling
+context, all output is captured -- nothing is passed to the existing filehandles.
 
 It is prototyped to take a subroutine reference as an argument. Thus, it
 can be called in block form:
@@ -467,7 +470,7 @@ scalar context on the return value, you must use the C<<< scalar >>> keyword.
      return scalar @list; # $count will be 3
    };
 
-Captures are normally done internally to an anonymous filehandle.  To
+Captures are normally done to an anonymous temporary filehandle.  To
 capture via a named file (e.g. to externally monitor a long-running capture),
 provide custom filehandles as a trailing list of option pairs:
 
@@ -479,6 +482,9 @@ The filehandles must be readE<sol>write and seekable.  Modifying the files or
 filehandles during a capture operation will give unpredictable results.
 Existing IO layers on them may be changed by the capture.
 
+When called in void context, C<<< capture >>> saves memory and time by
+not reading back from the capture handles.
+
 =head2 capture_stdout
 
    ($stdout, @result) = capture_stdout \&code;
@@ -515,6 +521,11 @@ properly ordered due to buffering.
 The C<<< tee >>> function works just like C<<< capture >>>, except that output is captured
 as well as passed on to the original STDOUT and STDERR.
 
+When called in void context, C<<< tee >>> saves memory and time by
+not reading back from the capture handles, except when the
+original STDOUT OR STDERR were tied or opened to a scalar
+handle.
+
 =head2 tee_stdout
 
    ($stdout, @result) = tee_stdout \&code;
@@ -548,19 +559,22 @@ properly ordered due to buffering.
 
 Portability is a goal, not a guarantee.  C<<< tee >>> requires fork, except on
 Windows where C<<< system(1, @cmd) >>> is used instead.  Not tested on any
-particularly esoteric platforms yet.
+particularly esoteric platforms yet.  See the
+L<CPAN Testers Matrix|http://matrix.cpantesters.org/?dist=Capture-Tiny>
+for test result by platform.
 
 =head2 PerlIO layers
 
 Capture::Tiny does it's best to preserve PerlIO layers such as ':utf8' or
-':crlf' when capturing.   Layers should be applied to STDOUT or STDERR I<before>
-the call to C<<< capture >>> or C<<< tee >>>.  This may not work for tied handles (see below).
+':crlf' when capturing (only for Perl 5.8.1+) .  Layers should be applied to
+STDOUT or STDERR I<before> the call to C<<< capture >>> or C<<< tee >>>.  This may not work
+for tied filehandles (see below).
 
 =head2 Modifying filehandles before capturing
 
 Generally speaking, you should do little or no manipulation of the standard IO
-handles prior to using Capture::Tiny.  In particular, closing, reopening,
-localizing or tying standard handles prior to capture may cause a variety of
+filehandles prior to using Capture::Tiny.  In particular, closing, reopening,
+localizing or tying standard filehandles prior to capture may cause a variety of
 unexpected, undesirable andE<sol>or unreliable behaviors, as described below.
 Capture::Tiny does its best to compensate for these situations, but the
 results may not be what you desire.
@@ -573,71 +587,67 @@ code within the captured block that depends on finding them closed will, of
 course, not find them to be closed.  If they started closed, Capture::Tiny will
 close them again when the capture block finishes.
 
-Note that this reopening will happen even for STDIN or a handle not being
+Note that this reopening will happen even for STDIN or a filehandle not being
 captured to ensure that the filehandle used for capture is not opened to file
 descriptor 0, as this causes problems on various platforms.
 
 B<Localized filehandles>
 
-If code localizes any of Perl's standard handles before capturing, the capture
-will affect the localized handles and not the original ones.  External system
-calls are not affected by localizing a handle in Perl and will continue
-to send output to the original handles (which will thus not be captured).
+If code localizes any of Perl's standard filehandles before capturing, the capture
+will affect the localized filehandles and not the original ones.  External system
+calls are not affected by localizing a filehandle in Perl and will continue
+to send output to the original filehandles (which will thus not be captured).
 
 B<Scalar filehandles>
 
 If STDOUT or STDERR are reopened to scalar filehandles prior to the call to
-C<<< capture >>> or C<<< tee >>>, then Capture::Tiny will override the output handle for the
-duration of the C<<< capture >>> or C<<< tee >>> call and then send captured output to the
-output handle after the capture is complete.  (Requires Perl 5.8)
+C<<< capture >>> or C<<< tee >>>, then Capture::Tiny will override the output filehandle for
+the duration of the C<<< capture >>> or C<<< tee >>> call and then, for C<<< tee >>>, send captured
+output to the output filehandle after the capture is complete.  (Requires Perl
+5.8)
 
 Capture::Tiny attempts to preserve the semantics of STDIN opened to a scalar
-reference.
+reference, but note that external processes will not be able to read from such
+a handle.  Capture::Tiny tries to ensure that external processes will read from
+the null device instead, but this is not guaranteed.
 
-B<Tied output handles>
+B<Tied output filehandles>
 
 If STDOUT or STDERR are tied prior to the call to C<<< capture >>> or C<<< tee >>>, then
 Capture::Tiny will attempt to override the tie for the duration of the
-C<<< capture >>> or C<<< tee >>> call and then send captured output to the tied handle after
+C<<< capture >>> or C<<< tee >>> call and then send captured output to the tied filehandle after
 the capture is complete.  (Requires Perl 5.8)
 
 Capture::Tiny may not succeed resending UTF-8 encoded data to a tied
-STDOUT or STDERR handle.  Characters may appear as bytes.  If the tied handle
+STDOUT or STDERR filehandle.  Characters may appear as bytes.  If the tied filehandle
 is based on L<Tie::StdHandle>, then Capture::Tiny will attempt to determine
-appropriate layers like C<<< :utf8 >>> from the underlying handle and do the right
+appropriate layers like C<<< :utf8 >>> from the underlying filehandle and do the right
 thing.
 
-B<Tied input handle>
+B<Tied input filehandle>
 
-Capture::Tiny attempts to preserve the semantics of tied STDIN, but this is not
-entirely stable or portable. For example:
-
-=over
-
-=item *
-
-Capturing or teeing with STDIN tied is broken on Windows
-
-=item *
-
-L<FCGI> has been reported as having a pathological tied handle implementation
-that causes fatal (and hard to diagnose) errors
-
-=back
+Capture::Tiny attempts to preserve the semantics of tied STDIN, but this
+requires Perl 5.8 and is not entirely predictable.  External processes
+will not be able to read from such a handle.
 
 Unless having STDIN tied is crucial, it may be safest to localize STDIN when
 capturing:
 
    my ($out, $err) = do { local *STDIN; capture { ... } };
 
-=head2 Modifying handles during a capture
+=head2 Modifying filehandles during a capture
 
 Attempting to modify STDIN, STDOUT or STDERR I<during> C<<< capture >>> or C<<< tee >>> is
 almost certainly going to cause problems.  Don't do that.
 
 =head2 No support for Perl 5.8.0
 
-It's just too buggy when it comes to layers and UTF-8.
+It's just too buggy when it comes to layers and UTF-8.  Perl 5.8.1 or later
+is recommended.
+
+=head2 Limited support for Perl 5.6
+
+Perl 5.6 predates PerlIO.  UTF-8 data may not be captured correctly.
 
 =head1 ENVIRONMENT
 
@@ -649,15 +659,6 @@ timeout with an error if the subprocesses are not ready to receive data within
 alternate timeout may be specified by setting the C<<< PERL_CAPTURE_TINY_TIMEOUT >>>
 environment variable.  Setting it to zero will disable timeouts.
 
-=head1 BUGS
-
-Please report any bugs or feature requests using the CPAN Request Tracker.
-Bugs can be submitted through the web interface at
-L<http://rt.cpan.org/Dist/Display.html?Queue=Capture-Tiny>
-
-When submitting a bug or request, please include a test-file or a patch to an
-existing test-file that illustrates the bug or desired feature.
-
 =head1 SEE ALSO
 
 This module was, inspired by L<IO::CaptureOutput>, which provides
@@ -759,7 +760,7 @@ L<Test::Output>
 
 =back
 
-=for :stopwords cpan testmatrix url annocpan anno bugtracker rt cpants kwalitee diff irc mailto metadata placeholders
+=for :stopwords cpan testmatrix url annocpan anno bugtracker rt cpants kwalitee diff irc mailto metadata placeholders metacpan
 
 =head1 SUPPORT
 
